@@ -1,5 +1,5 @@
-# Copyright (C) 2019-2024 Valéry Febvre
-# SPDX-License-Identifier: GPL-3.0-only or GPL-3.0-or-later
+# SPDX-FileCopyrightText: 2019-2025 Valéry Febvre
+# SPDX-License-Identifier: GPL-3.0-or-later
 # Author: Valéry Febvre <vfebvre@easter-eggs.com>
 
 from gettext import gettext as _
@@ -33,14 +33,31 @@ class ChapterItemWrapper(GObject.Object):
         self.emit('changed')
 
 
-class ChaptersListModel(Gtk.SortListModel):
+class ChaptersFilterListModel(Gtk.FilterListModel):
+    def __init__(self, model, filter_func):
+        self.filter = Gtk.CustomFilter()
+        self.filter_func = filter_func
+        self.filter.set_filter_func(self.filter_func)
+
+        super().__init__(model=model, filter=self.filter)
+
+    def invalidate_filter(self):
+        self.filter.set_filter_func(self.filter_func)
+
+    def invalidate_sort(self):
+        self.get_model().sorter.set_sort_func(self.get_model().sort_func)
+
+    def populate(self, chapters):
+        self.get_model().populate(chapters)
+
+
+class ChaptersSortListModel(Gtk.SortListModel):
     def __init__(self, sort_func):
         self.sorter = Gtk.CustomSorter()
         self.sort_func = sort_func
         self.sorter.set_sort_func(self.sort_func)
-        self.list_store = Gio.ListStore(item_type=ChapterItemWrapper)
 
-        super().__init__(model=self.list_store, sorter=self.sorter)
+        super().__init__(model=Gio.ListStore(item_type=ChapterItemWrapper), sorter=self.sorter)
 
     def invalidate_sort(self):
         self.sorter.set_sort_func(self.sort_func)
@@ -50,7 +67,7 @@ class ChaptersListModel(Gtk.SortListModel):
         for chapter in chapters:
             items.append(ChapterItemWrapper(chapter))
 
-        self.list_store.splice(0, self.list_store.props.n_items, items)
+        self.get_model().splice(0, self.get_model().props.n_items, items)
 
 
 class ChaptersList:
@@ -67,7 +84,7 @@ class ChaptersList:
         self.factory.connect('setup', self.on_factory_setup)
         self.factory.connect('unbind', self.on_factory_unbind)
 
-        self.list_model = ChaptersListModel(self.sort_func)
+        self.list_model = ChaptersFilterListModel(ChaptersSortListModel(self.sort_func), self.filter_func)
         self.model = Gtk.MultiSelection.new(self.list_model)
         self.selection_changed_handler_id = self.model.connect('selection-changed', self.on_selection_changed)
 
@@ -159,8 +176,7 @@ class ChaptersList:
         item.chapter = Chapter.get(item.chapter.id)
         item.emit_changed()
 
-        if reset:
-            self.card.info_box.refresh()
+        self.card.refresh(unread_chapters=reset, info=True)
         self.card.window.library.refresh_on_manga_state_changed(self.card.manga)
 
     def clear_selected_chapters(self, _action, _param, reset):
@@ -171,8 +187,7 @@ class ChaptersList:
             item.chapter = Chapter.get(item.chapter.id)
             item.emit_changed()
 
-        if reset:
-            self.card.info_box.refresh()
+        self.card.refresh(unread_chapters=reset, info=True)
         self.card.window.library.refresh_on_manga_state_changed(self.card.manga)
 
         self.card.leave_selection_mode()
@@ -197,6 +212,18 @@ class ChaptersList:
             # Init selection with clicked row (stored in self.selection_click_position)
             self.on_selection_changed(None, None, None)
 
+    def filter_func(self, item: ChapterItemWrapper) -> int:
+        if not self.card.manga.filters or not self.card.manga.filters.get('scanlators'):
+            # No scanlators filter
+            return True
+
+        if not item.chapter.scanlators:
+            # Chapter has no scanlators defined
+            # It is not filtered if 'Unknown' virtual scanlator does not belong to filter
+            return 'Unknown' not in self.card.manga.filters['scanlators']
+
+        return len(set(item.chapter.scanlators or []).difference(set(self.card.manga.filters['scanlators']))) > 0
+
     def get_selected_chapters_items(self):
         items = []
 
@@ -380,6 +407,8 @@ class ChaptersList:
                 item.emit_changed()
 
             self.card.window.activity_indicator.set_visible(False)
+
+            self.card.refresh(unread_chapters=True)
             self.card.window.library.refresh_on_manga_state_changed(self.card.manga)
         else:
             self.card.window.activity_indicator.set_visible(False)
@@ -395,7 +424,7 @@ class ChaptersList:
         This function gets two children and has to return:
         - a negative integer if the firstone should come before the second one
         - zero if they are equal
-        - a positive integer if the second one should come before the firstone
+        - a positive integer if the second one should come before the first one
         """
         if self.sort_order in ('asc', 'desc'):
             if item1.chapter.rank > item2.chapter.rank:
@@ -433,6 +462,8 @@ class ChaptersList:
         chapter.update(data)
 
         item.emit_changed()
+
+        self.card.refresh(unread_chapters=True)
         self.card.window.library.refresh_on_manga_state_changed(self.card.manga)
 
     def toggle_selected_chapters_read_status(self, _action, _gparam, read):
@@ -468,6 +499,8 @@ class ChaptersList:
 
                 self.card.leave_selection_mode()
                 self.card.window.activity_indicator.set_visible(False)
+
+                self.card.refresh(unread_chapters=True)
                 self.card.window.library.refresh_on_manga_state_changed(self.card.manga)
 
             def run_generator(func):
@@ -580,11 +613,11 @@ class ChaptersListRow(Gtk.Box):
             )
 
         self.title_label.set_label(self.chapter.title)
-        self.title_label.remove_css_class('dim-label')
+        self.title_label.remove_css_class('dimmed')
         self.title_label.remove_css_class('warning')
         if self.chapter.read:
             # Chapter reading ended
-            self.title_label.add_css_class('dim-label')
+            self.title_label.add_css_class('dimmed')
         elif self.chapter.last_page_read_index is not None:
             # Chapter reading started
             self.title_label.add_css_class('warning')
