@@ -1,5 +1,5 @@
-# Copyright (C) 2019-2024 Valéry Febvre
-# SPDX-License-Identifier: GPL-3.0-only or GPL-3.0-or-later
+# SPDX-FileCopyrightText: 2019-2025 Valéry Febvre
+# SPDX-License-Identifier: GPL-3.0-or-later
 # Author: Valéry Febvre <vfebvre@easter-eggs.com>
 
 import datetime
@@ -20,6 +20,8 @@ from komikku.models.database import update_rows
 from komikku.servers.utils import get_server_class_name_by_id
 from komikku.servers.utils import get_server_dir_name_by_id
 from komikku.servers.utils import get_server_module_name_by_id
+from komikku.utils import COVER_HEIGHT
+from komikku.utils import COVER_WIDTH
 from komikku.utils import get_cached_data_dir
 from komikku.utils import get_data_dir
 from komikku.utils import is_number
@@ -31,6 +33,7 @@ logger = logging.getLogger(__name__)
 
 class Manga:
     _chapters = None
+    _chapters_scanlators = None
     _server = None
 
     STATUSES = dict(
@@ -77,7 +80,7 @@ class Manga:
         data.update(dict(
             in_library=0,
             # Add fake last_read date: allows to display recently added manga at the top of the library
-            last_read=datetime.datetime.utcnow(),
+            last_read=datetime.datetime.now(datetime.UTC),
         ))
 
         # Long strip detection (Webtoon)
@@ -177,6 +180,34 @@ class Manga:
 
         return self._chapters
 
+    @property
+    def chapters_scanlators(self):
+        if self._chapters_scanlators is None:
+            db_conn = create_db_connection()
+
+            rows = db_conn.execute('SELECT DISTINCT scanlators, count(*) FROM chapters WHERE manga_id = ? GROUP BY scanlators', (self.id,))
+
+            scanlators = {}
+            for row in rows:
+                if not row[0]:  # None or []
+                    # Use 'Unknown' as virtual scanlator for chapters without scanlators defined
+                    row = (['Unknown'], row[1])
+
+                for scanlator in row[0]:
+                    if scanlator not in scanlators:
+                        scanlators[scanlator] = {
+                            'name': scanlator,
+                            'count': 0,
+                        }
+
+                    scanlators[scanlator]['count'] += row[1]
+
+            self._chapters_scanlators = list(scanlators.values()) or None
+
+            db_conn.close()
+
+        return self._chapters_scanlators
+
     @property
     def class_name(self):
         return get_server_class_name_by_id(self.server_id)
@@ -252,34 +283,8 @@ class Manga:
         return self._server
 
     def _save_cover(self, url):
-        if url is None:
-            return
-
-        # If cover has already been retrieved
-        # Check first if it has changed using ETag
-        current_etag = None
-        cover_etag_fs_path = os.path.join(self.path, 'cover.etag')
-        if os.path.exists(cover_etag_fs_path):
-            with open(cover_etag_fs_path, 'r') as fp:
-                current_etag = fp.read()
-
-        # Save cover image file
-        try:
-            cover_data, etag, _rtime = self.server.get_manga_cover_image(url, current_etag)
-        except Exception:
-            return
-        if cover_data is None:
-            return
-
-        cover_fs_path = os.path.join(self.path, 'cover.jpg')
-        with open(cover_fs_path, 'wb') as fp:
-            fp.write(cover_data)
-
-        if etag:
-            with open(cover_etag_fs_path, 'w') as fp:
-                fp.write(etag)
-        elif os.path.exists(cover_etag_fs_path):
-            os.remove(cover_etag_fs_path)
+        # Covers in landscape format are converted to portrait format
+        return self.server.save_image(url, self.path, 'cover', COVER_WIDTH, COVER_HEIGHT)
 
     def add_in_library(self):
         tmp_path = self.path
@@ -321,24 +326,62 @@ class Manga:
 
         db_conn = create_db_connection()
 
-        order = 'ASC' if direction == 1 else 'DESC'
         op = '>' if direction == 1 else '<'
+        order = 'ASC' if direction == 1 else 'DESC'
+        if self.filters and self.filters.get('scanlators'):
+            # Chapters must be filtered by scanlators (some scanlators are excluded)
+            excluded_scanlators = self.filters['scanlators']
+
+            # Subquery to get IDs of not filtered chapters
+            scanlators_subquery = f"""
+                SELECT DISTINCT c.id
+                FROM chapters c, json_each(scanlators)
+                WHERE json_each.value NOT IN ("{'", "'.join(excluded_scanlators)}") AND c.manga_id = {self.id}
+            """
+            if 'Unknown' not in excluded_scanlators:
+                # Add chapters without scanlators defined
+                scanlators_subquery += f"""
+                    UNION
+                    SELECT id FROM chapters WHERE (scanlators IS NULL OR scanlators->0 IS NULL) AND manga_id = {self.id}
+                """
+        else:
+            scanlators_subquery = None
 
         if self.sort_order in ('asc', 'desc', None):
-            row = db_conn.execute(
-                f'SELECT * FROM chapters WHERE manga_id = ? AND rank {op} ? ORDER BY rank {order}',
-                (self.id, chapter.rank)
-            ).fetchone()
+            if scanlators_subquery:
+                row = db_conn.execute(
+                    f'SELECT * FROM chapters WHERE manga_id = ? AND id IN ({scanlators_subquery}) AND rank {op} ? ORDER BY rank {order}',
+                    (self.id, chapter.rank)
+                ).fetchone()
+            else:
+                row = db_conn.execute(
+                    f'SELECT * FROM chapters WHERE manga_id = ? AND rank {op} ? ORDER BY rank {order}',
+                    (self.id, chapter.rank)
+                ).fetchone()
+
         elif self.sort_order in ('date-asc', 'date-desc'):
-            row = db_conn.execute(
-                f'SELECT * FROM chapters WHERE manga_id = ? AND date {op} ? ORDER BY date {order}, id {order}',
-                (self.id, chapter.date)
-            ).fetchone()
+            if scanlators_subquery:
+                row = db_conn.execute(
+                    f'SELECT * FROM chapters WHERE manga_id = ? AND id IN ({scanlators_subquery}) AND date {op} ? ORDER BY date {order}, id {order}',
+                    (self.id, chapter.date)
+                ).fetchone()
+            else:
+                row = db_conn.execute(
+                    f'SELECT * FROM chapters WHERE manga_id = ? AND date {op} ? ORDER BY date {order}, id {order}',
+                    (self.id, chapter.date)
+                ).fetchone()
+
         elif self.sort_order in ('natural-asc', 'natural-desc'):
-            row = db_conn.execute(
-                f'SELECT * FROM chapters WHERE manga_id = ? AND title {op} ? COLLATE natsort ORDER BY title {order}, id {order}',
-                (self.id, chapter.title)
-            ).fetchone()
+            if scanlators_subquery:
+                row = db_conn.execute(
+                    f'SELECT * FROM chapters WHERE manga_id = ? AND id IN ({scanlators_subquery}) AND title {op} ? COLLATE natsort ORDER BY title {order}, id {order}',
+                    (self.id, chapter.title)
+                ).fetchone()
+            else:
+                row = db_conn.execute(
+                    f'SELECT * FROM chapters WHERE manga_id = ? AND title {op} ? COLLATE natsort ORDER BY title {order}, id {order}',
+                    (self.id, chapter.title)
+                ).fetchone()
 
         db_conn.close()
 
@@ -508,9 +551,10 @@ class Manga:
                         logger.info('[UPDATE] {0} ({1}): Add new chapter {2}'.format(self.name, self.server_id, chapter_data['title']))
 
             if len(recent_chapters_ids) > 0 or nb_deleted_chapters > 0:
-                data['last_update'] = datetime.datetime.utcnow()
+                data['last_update'] = datetime.datetime.now(datetime.UTC)
 
             self._chapters = None
+            self._chapters_scanlators = None
 
             # Store old path
             old_path = self.path
@@ -597,21 +641,12 @@ class Chapter:
     @property
     def number(self):
         """ Returns chapter number"""
-        num = None
         if self.num and is_number(self.num):
-            num = self.num
+            return remove_number_leading_zero(self.num)
 
-        elif is_number(self.slug):
-            # The server doesn't provide chapter numbers, but slug is a number
-            # So we use it as fallback even if we're not sure it's a chapter number
-            num = self.slug
+        logger.warning(f'{self.manga.name} serie ({self.manga.server_id}) do not support tracking (no chapter num)?')
 
-        if num:
-            num = remove_number_leading_zero(num)
-        else:
-            logger.warning('{0} server do not support tracking (no chapter num)?'.format(self.manga.server_id))
-
-        return num
+        return None
 
     @property
     def path(self):
