@@ -1,12 +1,12 @@
-# Copyright (C) 2019-2024 Valéry Febvre
-# SPDX-License-Identifier: GPL-3.0-only or GPL-3.0-or-later
+# SPDX-FileCopyrightText: 2019-2025 Valéry Febvre
+# SPDX-License-Identifier: GPL-3.0-or-later
 # Author: Valéry Febvre <vfebvre@easter-eggs.com>
 
 # Madara – WordPress Theme for Manga
 
 # Supported servers:
 # 24hRomance [EN] (disabled)
-# 3asq [AR]
+# 3asq [AR] (disabled)
 # AkuManga [AR] (disabled)
 # Aloalivn [EN] (disabled)
 # Apoll Comics [ES]
@@ -17,7 +17,7 @@
 # Best Manga [RU] (disabled)
 # Fr-Scan (Id frdashscan) [FR]
 # Hunlight Scans [EN] (disabled)
-# Leomanga [ES] (disabled)
+# Leomanga [ES]
 # Leviatanscans [EN] (disabled)
 # Manga-Scantrad [FR]
 # Mangas Origines [FR] (disabled)
@@ -25,6 +25,7 @@
 # Manhuaus [EN]
 # Manhwa Hentai [EN]
 # Phoenix Fansub [ES] (disabled)
+# Poseidon Scans [FR]
 # Reaperscans [AR/FR/ID/TR]
 # Submanga [ES] (disabled)
 # ToonGod [EN]
@@ -181,7 +182,7 @@ class Madara(Server):
 
                 if status in ('Completed', 'Terminé', 'Completé', 'Completo', 'Finalizado', 'Concluído', 'Tamamlandı', 'مكتملة', 'Закончена'):
                     data['status'] = 'complete'
-                elif status in ('OnGoing', 'En Cours', 'En cours', 'Updating', 'Devam Ediyor', 'Em Lançamento', 'Em andamento', 'En Emision', 'مستمرة', 'Продолжается', 'Выпускается'):
+                elif status in ('OnGoing', 'En Cours', 'En cours', 'Updating', 'Devam Ediyor', 'Em Lançamento', 'Em andamento', 'En Emision', 'En emisión', 'En Emisión', 'مستمرة', 'Продолжается', 'Выпускается'):
                     data['status'] = 'ongoing'
                 elif status in ('Canceled', 'Cancelada'):
                     data['status'] = 'suspended'
