@@ -1,5 +1,5 @@
-# Copyright (C) 2020-2024 GrownNed
-# SPDX-License-Identifier: GPL-3.0-only or GPL-3.0-or-later
+# SPDX-FileCopyrightText: 2020-2024 GrownNed
+# SPDX-License-Identifier: GPL-3.0-or-later
 # Author: GrownNed <grownned@gmail.com>
 
 from bs4 import BeautifulSoup
@@ -15,17 +15,18 @@ from komikku.utils import is_number
 
 class Readmanga(Server):
     id = 'readmanga'
-    name = 'Read Manga'
+    name = 'Usagi (Read Manga)'
     lang = 'ru'
     is_nsfw = True
 
-    base_url = 'https://zz.readmanga.io'
+    base_url = 'https://web.usagi.one'
+    logo_url = base_url + '/static/favicon-8d70559a59e754c642d8ccf73bb55d47.ico'
     search_url = base_url + '/search/advancedResults'
     manga_url = base_url + '/{0}'
-    chapter_url = manga_url + '/{1}?mtr=1'
+    chapter_url = manga_url + '/{1}'
 
-    pages_js_start = 'rm_h.readerDoInit'
-    pages_js_offset = 18
+    pages_js_start = 'rm_h.readerInit'
+    pages_js_offset = 16
 
     def __init__(self):
         if self.session is None:
@@ -229,7 +230,7 @@ class Readmanga(Server):
             a_element = element.select_one('.desc > h3 > a')
             results.append(dict(
                 name=a_element.text.strip(),
-                slug=a_element.get('href')[1:],
+                slug=a_element.get('href').split('/')[-1],
                 cover=element.select_one('.img img.img-fluid').get('data-original'),
             ))
 
@@ -243,6 +244,7 @@ class Allhentai(Readmanga):
     # FIXME: requires to be logged in
 
     base_url = 'https://20.allhen.online'
+    logo_url = base_url + '/static/favicon-dcc74ed15e02e1f03481426ba63716e9.ico'
     search_url = base_url + '/search/advanced'
     manga_url = base_url + '/{0}'
     chapter_url = manga_url + '/{1}?mtr=1'
@@ -252,6 +254,7 @@ class Mintmanga(Readmanga):
     id = 'mintmanga:readmanga'
     name = 'Mint Manga'
     is_nsfw = True
+    status = 'disabled'
 
     # 16
     base_url = 'https://2.mintmanga.one'
@@ -266,6 +269,7 @@ class Selfmanga(Readmanga):
     is_nsfw = True
 
     base_url = 'https://selfmanga.live'
+    logo_url = base_url + '/static/favicon-29c351d867912538a132f31da35d87aa.ico'
     search_url = base_url + '/search/advancedResults'
     manga_url = base_url + '/{0}'
     chapter_url = manga_url + '/{1}?mtr=1'
